################################################################################
# trieda ColorPalette - funkcie:
#    addColorsFromFile
#       -vstup: subor (file) s farebnou skalou
#       -vystup: -
#       -nastavi colors na farby zo suboru file
#    addColor
#       -vstup: (num) cislo farby, (color) RGB hodnota farby
#       -vystup: -
#       -prida farbu s cislom num do zoznamu a nastavy jej RGB hodnotu 
#       -na color. Ak sa tam farba nachadza,zmeni len RGB hodnotu 
#    removeColor
#       -vstup: (num) cislo farby
#       -vystup: -
#       -ak sa farba s cislom num nachadza v colors, odstrani ju
#    getNumbersOfColors
#       -vstup: -
#       -vystup: mnozina cisel farieb v tejto skale
#       -vrati mnozinu cisel farieb v tejto skale
#    colorIsInPalette
#       -vstup: (num) cislo farby
#       -vystup: true alebo false
#       -vrati true ak sa farba s cislom num nachadza v skale, inak vrati false
#    getColor
#       -vstup: (num) cislo farby
#       -vystup: (color) RGB hodnota farby
#       -vrati RGB hodnotu farby s cislom num, ak sa tam taka farba nenachadza,
#        vrati None 
#    getNearestNumToRGBColor
#       -vstup: (color) RGB hodnota
#       -vystup: (num) cislo farby
#       -vrati cislo farby, ktora je najblizsie k farbe color 
#        (3D priestor - klasicka vzdialenost)
################################################################################
 
from PySide import QtCore, QtGui
import os
from queue import *

class ColorPalette():
    def __init__(self):
        self.colors = {}
        self.database = {}
        
    def addColorsFromFile(self,paletteName):
        path = os.path.abspath(__file__) #cesta k prave vykonavanemu programu
        path = path.rstrip("ColorPalette.py") #odstranenie nevhodneho konca
        file = path + "scales/" + str(paletteName) +".txt" #prifarenie vhodneho
        #print(file)
        
        ins = open(file)
        a = ins.read()
        ins.close;
        a = a.split('\n')
        for i in range(0,len(a)):
            if len(a[i]) != 0: 
                pom = a[i].split()
                self.colors[(pom[0], paletteName)] = int(pom[1])                
        
    def addColor(self,id,color):
        self.colors[id] = color
        self.database = {}
        
    def removeColor(self,id):
        try:
            del self.colors[id]
            self.database = {}
        except KeyError:
            pass
            
    def clear(self):
        self.colors = {}
        self.database = {}        
        
    def getNumbersOfColors(self):
        return self.colors.keys()    
    
    def colorIsInPalette(self,id):
        if id in self.colors.keys():
            return True
        return False                
        
    def getColor(self,id):
        if id in self.colors.keys():
            return self.colors[id] 
        else: return None    
        
    def getNearestNumToRGBColor(self,color):
        #print(self.colors)
    
        if color in self.database.keys():
            return self.database[color]
        
        #farba zo vstupu
        color1 = QtGui.QColor(color)
        r1 = color1.red()
        g1 = color1.green()
        b1 = color1.blue()    
        min = 256*256*256+47
        minK = "."    
        for i in self.colors.keys(): 
            color2 =  QtGui.QColor(self.colors[i])
            r2 = color2.red()
            g2 = color2.green()
            b2 = color2.blue()
            vz = (r1-r2)*(r1-r2) + (g1-g2)*(g1-g2) + (b1-b2)*(b1-b2)
            #print(str(i)+" "+str(vz))
            if vz < min:
                min = vz
                minK = i
        self.database[color] = minK
        return minK  
        
    def find_distance(self,id1,id2):
        rgb1 = self.getColor(id1)
        rgb2 = self.getColor(id2)
        color1 = QtGui.QColor(rgb1)
        color2 = QtGui.QColor(rgb2)
        r1 = color1.red()
        g1 = color1.green()
        b1 = color1.blue()
        r2 = color2.red()
        g2 = color2.green()
        b2 = color2.blue()
        vz = (r1-r2)*(r1-r2) + (g1-g2)*(g1-g2) + (b1-b2)*(b1-b2)
        return vz    
        
    def calculateD(self): # vypocita pri akej vzdialenosti je graf spojity
        min = -1
        max = 3*256*256
        minStupen = 340934899
        maxStupen = -1
        
        while (max-min > 1):
            stred = (max+min)//2
            lub = self.colors.popitem() #vybrala som lubovolny prvok
            self.colors[lub[0]] = lub[1] #musim ho zas vratit
            key = lub[0]
            
            najdene = set()
            q = Queue()
            q.put(key)
            najdene.add(key)
            
            minStupen = 340934899
            maxStupen = -1
                        
            while not q.empty():
                p = q.get()
                stupen = 0
                for num in self.colors.keys():
                    vz = self.find_distance(num,p)
                    if (vz < stred) and (vz > 0):
                        stupen = stupen+1
                    if (vz < stred) and not(num in najdene):
                        najdene.add(num)
                        q.put(num)
                        
                if stupen > maxStupen: maxStupen = stupen
                if stupen < minStupen: minStupen = stupen
                        
            if len(najdene) == len(self.colors.keys()):
                max = stred
            else: min = stred
            
        print(str(minStupen)+" "+str(maxStupen))    
        return max             
        
        
if __name__ == '__main__':	
	
    import sys
    
    paleta = ColorPalette()
    paleta.setFromFile("ariadna")
    vz = paleta.calculateD()
    print(str(vz))
    
                                      
                                       