from PySide import QtCore, QtGui
from Constants import *
from ColorPalette import *

class ButtonGrid(QtGui.QWidget):
    def __init__(self, buttons):
        super(ButtonGrid,self).__init__()
        self.Layout = QtGui.QGridLayout()
        for i in range(0,len(buttons)):
            y = i // 10
            x = i % 10
            self.Layout.addWidget(buttons[i],y,x)
        self.setLayout(self.Layout) 
        
class ColorButton(QtGui.QPushButton):
    def __init__(self,color):
        super(ColorButton,self).__init__()
        self.color = color
        
    def getColor(self):
        return self.color        

class NumButton(QtGui.QPushButton):
    def __init__(self,icon,text):
        super(NumButton,self).__init__(icon,text)
        self.num = 0 
        self.col = 0
        self.setAutoDefault(False)   
    
    def setNumber(self,col):
        self.col = col
        
    def getNumber(self):
        return self.col 
        
    def setSymbol(self,num):
        self.num = num
        
    def getSymbol(self):
        return self.num          


class SymbolDialog(QtGui.QDialog):
    def __init__(self,symbol):
        super(SymbolDialog, self).__init__()
        self.symbol = symbol
        self.buttons = []
        self.SymbolsLayout = QtGui.QGridLayout()
        
        for i in range(MAX_COLORS_COUNT()):
            filename = "symbols/s"+'{:0>4}'.format(str(i))+".jpg"
            button = NumButton(QtGui.QIcon(filename),"")
            button.setCheckable(True)
            button.setSymbol(i)
            if(self.symbol == i): button.setChecked(True)
            button.clicked[bool].connect(self.changeSymbol)
            self.buttons.append(button)
            y = i // 12
            x = i % 12
            self.SymbolsLayout.addWidget(self.buttons[i],y,x)
            
        self.ButtonLayout = QtGui.QHBoxLayout()
        self.okButton = QtGui.QPushButton("Ok")
        QtCore.QObject.connect(self.okButton, QtCore.SIGNAL('clicked()'), self.ok)
        self.cancelButton = QtGui.QPushButton("Cancel")
        QtCore.QObject.connect(self.cancelButton, QtCore.SIGNAL('clicked()'), self.cancel)
        self.ButtonLayout.addWidget(self.okButton)
        self.ButtonLayout.addWidget(self.cancelButton)
        
        self.Layout = QtGui.QVBoxLayout()
        self.Layout.addLayout(self.SymbolsLayout)
        self.Layout.addLayout(self.ButtonLayout)
               
        self.setLayout(self.Layout)              
            
    def changeSymbol(self):
        source = self.sender()
        num = source.getSymbol()
        if(self.symbol != num):
            self.buttons[self.symbol].setChecked(False)
            self.buttons[num].setChecked(True)
        self.symbol = num  
        
    def ok(self):
        self.accept()
        
    def cancel(self):
        self.reject()
        
    def getSymbol(self):
        return self.symbol 
        
        
        
        
class ColorDialog(QtGui.QDialog): 
    def __init__(self,color):
        super(ColorDialog, self).__init__()
        self.color = color  #= ((cislo, znacka), rgb) ak (None,None) potom = bez vyplne
        
        self.colorPaletts = {}
        for name in NAMES():
            print("nacitavam "+name)
            self.colorPaletts[name] = ColorPalette()
            self.colorPaletts[name].clear()
            self.colorPaletts[name].addColorsFromFile(name)
      
        self.buttons = []
        
        #aktualna farba (vybrata na vstupe)
        self.actualColorLabel = QtGui.QLabel("Actual color:",self)
        self.actualColorImage = QtGui.QLabel(self)
        self.actualColorNumBrand = QtGui.QLabel(self)
        
        self.updateActualColor()
            
        self.actualLayout = QtGui.QHBoxLayout()
        self.actualLayout.addWidget(self.actualColorLabel)
        self.actualLayout.addWidget(self.actualColorImage)
        self.actualLayout.addWidget(self.actualColorNumBrand)
        self.actualLayout.addWidget(QtGui.QWidget())
        self.actualLayout.setStretch(3,100)
        
        #bez vyplne
        self.NoFillButton = ColorButton((None,None))
        image = QtGui.QImage(1,1, QtGui.QImage.Format_ARGB32)
        image.setPixel(0,0, (QtGui.QColor(255,255,255)).rgb())
        image = image.scaled(15*image.size())
        icon = QtGui.QIcon(QtGui.QPixmap.fromImage(image))
        self.NoFillButton.setIcon(icon)
        self.NoFillButton.setText("No fill")
        QtCore.QObject.connect(self.NoFillButton, QtCore.SIGNAL('clicked()'), self.changeColor)
        
        #vyber palety + paleta
        self.brandLabel = QtGui.QLabel("Brand:")
        self.brandEdit = QtGui.QComboBox(self)
        i = 0
        for name in NAMES():
            self.brandEdit.insertItem(i,name)
            i = i+1
        self.brandEdit.setCurrentIndex(0)
        QtCore.QObject.connect(self.brandEdit,QtCore.SIGNAL('currentIndexChanged(int)'),self.updateButtons)  
        
        self.brandLayout = QtGui.QHBoxLayout()
        self.brandLayout.addWidget(self.brandLabel)
        self.brandLayout.addWidget(self.brandEdit)
        self.brandLayout.addWidget(QtGui.QWidget())
        self.brandLayout.setStretch(2,100)
        
        self.buttonsArea = QtGui.QScrollArea()
        self.updateButtons()
        
        #ok cancle buttons
        self.okButton = QtGui.QPushButton("Ok")
        QtCore.QObject.connect(self.okButton, QtCore.SIGNAL('clicked()'), self.ok)
        self.cancelButton = QtGui.QPushButton("Cancel")
        QtCore.QObject.connect(self.cancelButton, QtCore.SIGNAL('clicked()'), self.cancel)
        self.buttonLayout = QtGui.QHBoxLayout()
        self.buttonLayout.addWidget(QtGui.QWidget())
        self.buttonLayout.addWidget(self.okButton)
        self.buttonLayout.addWidget(self.cancelButton)
        self.buttonLayout.setStretch(0,100)
        
        #cely layout
        self.layout = QtGui.QVBoxLayout()
        self.layout.addLayout(self.actualLayout)
        self.layout.addWidget(self.NoFillButton)
        self.layout.addLayout(self.brandLayout)
        self.layout.addWidget(self.buttonsArea)
        self.layout.addLayout(self.buttonLayout)
        
        self.setLayout(self.layout)            
        
    def updateActualColor(self):                  
        image = QtGui.QImage(1,1, QtGui.QImage.Format_ARGB32)
        if self.color[0] == None:
            image.setPixel(0,0, (QtGui.QColor(255,255,255)).rgb())
        else:    
            image.setPixel(0,0, self.color[1])
        image = image.scaled(15*image.size())
        self.actualColorImage.setPixmap(QtGui.QPixmap.fromImage(image))    
        
        if self.color[0] == None:
            self.actualColorNumBrand.setText("No fill")
        else:
            self.actualColorNumBrand.setText(str(self.color[0][0])+" "+str(self.color[0][1]))
            
    def updateButtons(self):           
        self.buttons = []

        name = self.brandEdit.currentText() 
        for id in self.colorPaletts[name].getNumbersOfColors():
            button = ColorButton((id,self.colorPaletts[name].getColor(id)))
            image = QtGui.QImage(1,1, QtGui.QImage.Format_ARGB32)
            image.setPixel(0,0, self.colorPaletts[name].getColor(id))
            image = image.scaled(20*image.size())
            button.setIcon(QtGui.QIcon(QtGui.QPixmap.fromImage(image)))
            button.setText(str(id[0]))
            QtCore.QObject.connect(button, QtCore.SIGNAL('clicked()'), self.changeColor)
            self.buttons.append(button)
            
        BGrid = ButtonGrid(self.buttons)  
        
        self.buttonsArea.setWidget(BGrid)
      
             
    def changeColor(self):
        source = self.sender()
        self.color = source.getColor()
        self.updateActualColor()
        
    def ok(self):
        self.accept()
        
    def cancel(self):
        self.reject()
        
    def getColor(self):
        return self.color
        
                          