import os                                    # PREROBIT !!!!!!!!!!!!!!!!!!!!!
import shutil
import copy
from PySide import QtCore, QtGui
from abc import *
from PatternLabel import *
from SettingsWidget import *
from LeftWidget import *
from CalculatingMethods import *
from Constants import *
from Pattern import*
from ColorPalette import *

def ceil(a,b):
    if a%b == 0: return int(a/b)
    else: return int((a/b)+1)

class AbstractPatternWidget(QtGui.QWidget):
    __metaclass__ = ABCMeta

    def __init__(self):
        super(AbstractPatternWidget, self).__init__()
        
        self.modified = 0
        
        self.states = []   #stavy prace na predlohe
        self.indexToStates = 0 #oznacuje miesto, kam sa ma najblizsie zapisat novy stav
        self.indexUndo = 0 #kolko undo sme spravili a teda kolko vieme spravit redo 
        
        self.pattern = Pattern(0,0)
        self.selected = []
        
        self.colorPaletts = {}
        for name in NAMES():
            print("nacitavam "+name)
            self.colorPaletts[name] = ColorPalette()
            self.colorPaletts[name].clear()
            self.colorPaletts[name].addColorsFromFile(name)
        
        #up layout
        self.zoomLabel1 = QtGui.QLabel("zoom:",self)
        self.zoomSlider = QtGui.QSlider(QtCore.Qt.Horizontal)
        self.zoomSlider.setMaximumWidth(100)
        self.zoomSlider.setMinimumWidth(100)
        self.zoomSlider.setMinimum(1)
        self.zoomSlider.setMaximum(40)
        self.zoomSlider.setValue(10)
        QtCore.QObject.connect(self.zoomSlider, QtCore.SIGNAL('valueChanged(int)'),self.setZoom)
        self.zoomLabel2 = QtGui.QLabel("10",self) 
        
        #down layout
        
        self.allButton = QtGui.QPushButton("",self)
        self.allButton.setMaximumWidth(20)
        self.allButton.setMaximumHeight(20)
        self.Xaxis = QtGui.QWidget(self)
        self.Yaxis = QtGui.QWidget(self)
        
        self.patternLabel = PatternLabel(self)
        self.patternScrollArea = QtGui.QScrollArea(self)
        self.patternScrollArea.setBackgroundRole(QtGui.QPalette.Dark)
        self.patternScrollArea.setWidget(self.patternLabel)
        
        undo = QtGui.QIcon(QtGui.QPixmap.fromImage(QtGui.QImage("undo.png")))
        self.undoButton = QtGui.QPushButton(undo,"",self)
        self.undoButton.setEnabled(False)
        QtCore.QObject.connect(self.undoButton, QtCore.SIGNAL('clicked()'),self.undo)
        redo = QtGui.QIcon(QtGui.QPixmap.fromImage(QtGui.QImage("redo.png")))
        self.redoButton = QtGui.QPushButton(redo,"",self)
        self.redoButton.setEnabled(False)
        QtCore.QObject.connect(self.redoButton, QtCore.SIGNAL('clicked()'),self.redo)
        
        self.editLayout = QtGui.QHBoxLayout()
        self.editLayout.addWidget(self.undoButton)
        self.editLayout.addWidget(self.redoButton)
        
        self.zoomLayout = QtGui.QGridLayout()
        self.zoomLayout.addWidget(self.zoomLabel1,0,0)
        self.zoomLayout.addWidget(self.zoomSlider,0,1)
        self.zoomLayout.addWidget(self.zoomLabel2,0,2) 
        self.zoomLayout.setColumnMinimumWidth(2,15)              
        
        self.upLayout = QtGui.QGridLayout()
        self.upLayout.addWidget(QtGui.QWidget(),0,1)
        self.upLayout.addLayout(self.editLayout,0,2)
        self.upLayout.addLayout(self.zoomLayout,0,3)
        self.upLayout.setColumnStretch(1,200)
        
        self.rightLayout = QtGui.QGridLayout()
        self.rightLayout.addWidget(self.allButton,0,0)
        self.rightLayout.addWidget(self.Xaxis,0,1)
        self.rightLayout.addWidget(self.Yaxis,1,0)
        self.rightLayout.addWidget(self.patternScrollArea,1,1)
        self.rightLayout.setColumnMinimumWidth(0,10)
        self.rightLayout.setRowMinimumHeight(0,10)
        self.rightLayout.setColumnStretch(1,200)
        self.rightLayout.setRowStretch(1,200)
        
        self.downLayout = QtGui.QGridLayout()
        self.downLayout.addLayout(self.rightLayout,0,1)
        
        self.downLayout.setColumnStretch(1,200)
        self.downLayout.setColumnMinimumWidth(0,250)
        
        self.layout = QtGui.QGridLayout(self)
        self.layout.addLayout(self.upLayout,0,0)
        self.layout.addLayout(self.downLayout,1,0)
        
        self.layout.setRowStretch(1,200)
        self.layout.setRowMinimumHeight(0,2)
        
    def saveState(self):
        if self.pattern.getWidth()*self.pattern.getHeight() > 0:
            pattern = copy.deepcopy(self.pattern) 
            if self.indexToStates == len(self.states):
                self.states.append((pattern, self.selected ))
            else: self.states[self.indexToStates] = (pattern, self.selected)
            self.indexToStates = self.indexToStates + 1
            self.indexUndo = 0
            self.redoButton.setEnabled(False)
            self.undoButton.setEnabled(True)
        
    def openState(self,number):
        self.pattern = self.states[number][0]
        self.selected = self.states[number][1]
        self.patternLabel.setSelected(self.selected)
        self.updatePattern()  
                       
        
    def setZoom(self):
        self.patternLabel.SquareSize = self.zoomSlider.value()
        self.zoomLabel2.setText(str(self.zoomSlider.value()))
        self.repaint()         
    
    def Export(self,file): #zmenit velkost textu....
        path = file.split("/")
        name = path[len(path)-1]
        #print(name)
    
        output = open(str(file)+".tex","w")
        
        output.write("\\documentclass[a4paper]{article}\n")
        output.write("\\usepackage[slovak]{babel}\n")
        output.write("\\usepackage{tabulary}\n")
        output.write("\\usepackage{rotating}\n")
        output.write("\\usepackage[utf8x]{inputenc} % pre pdflatex\n")
        output.write("\\usepackage{epsfig} %pre pouzivanie obrazkov typu eps, s nizsie konverziou aj s pdf\n")
        output.write("\\usepackage[left=1.5cm,right=1.5cm,top=1.5cm,bottom=1.5cm]{geometry}\n")
        output.write("\n\\begin{document} %%%% samotny dokument zacina tu %%%%\n  \\ \\\ \n")
        
        output.write("{\\bf Legenda:}\n \\ \\\ \n  \\ \\\ \n")
        
        output.write("{\n \\renewcommand{\\arraystretch}{2}\n")
        output.write("\\begin{tabular}{|c|c|c|c|}\n\\hline\n")
        
        output.write("symbol & cislo & znacka & pradien \\\ \\hline \n")
        
        for col in self.pattern.getColors():
            count = self.pattern.getColorCount(col)
            countP = int((count+1)/SQUARECOUNT())+1
            Symbol = "s"+'{:0>4}'.format(str(self.pattern.getColorSymbol(col)))+".jpg"
            
            output.write("\\includegraphics[scale=0.41]{"+str(name)+"-symbols/"+Symbol+"} & "+str(col[0])+" & "+str(col[1])+" & "+str(countP)+" \\\ \\hline\n")
            
        output.write("\\end{tabular}\n")    
        output.write("}\n")
        
        output.write("\\newpage\n")
        
        #zacina samotna predloha( na kazdej strane je XxY stvorcekov)
        output.write("\\tiny{\n")
        Y = 130
        X = 80
        for y in range(0,ceil(self.pattern.getHeight(),Y)):
            for x in range(0,ceil(self.pattern.getWidth(),X)):
                output.write("{\n \\renewcommand{\\arraystretch}{0}\n")
                #kazda tabulka bude mat X stlpcov okrem možno poslednej 
                width = min(X*(x+1),self.pattern.getWidth()) - X*x
                height = min(Y*(y+1),self.pattern.getHeight()) - Y*y
                output.write("\\begin{tabular}{")
                for i in range(0,width+1): 
                    output.write("@{}c@{}")
                output.write("}\n") 
                for i in range(X*x,X*x+width): #prvy riadok tabulky
                    if (i+1)%10 == 0: 
                        output.write(" &\n\\multicolumn{1}{@{}c@{}}{\n     \\begin{sideways}\n          "+str(i+1)+" \\,\n     \\end{sideways}}\n")
                    else: output.write(" &")
                output.write("\\\ \n")
                      
                for i in range(Y*y,Y*y+height): #ostatne riadky ku
                    if (i+1)%10 == 0: output.write(str(i+1))
                    output.write(" &")
                    for j in range(X*x,X*x+width-1):
                        print(j)
                        sym = self.pattern.getColorSymbol(self.pattern.getSquare(j,i))
                        if sym == None: Symbol = "s0000.jpg"
                        else: Symbol = "s"+'{:0>4}'.format(str(sym))+".jpg"          
                        output.write("\\includegraphics[scale=0.41]{"+str(name)+"-symbols/"+Symbol+"} & ")
                    print(str(30*x+width-1)+"aaaaa")    
                    Symbol = "s"+'{:0>4}'.format(str(self.pattern.getColorSymbol(self.pattern.getSquare(X*x+width-1,i))))+".jpg"    
                    output.write("\\includegraphics[scale=0.41]{"+str(name)+"-symbols/"+Symbol+"}\\\ \n")
                
                output.write("\\end{tabular}\n")    
                output.write("}\n")
                
                output.write("\\newpage\n")
        
        output.write("}\n")
        output.write("\\end{document}\n")
        
    def undo(self):
        if self.indexUndo == 0:
            self.saveState()
            self.indexToStates = self.indexToStates - 1 # lebo save jedno pridala
        self.indexUndo = self.indexUndo+1    
        self.indexToStates = self.indexToStates - 1
        if self.indexToStates == 0: self.undoButton.setEnabled(False)
        self.openState(self.indexToStates)
        self.redoButton.setEnabled(True)
        
    def redo(self):
        self.indexUndo = self.indexUndo - 1
        self.indexToStates = self.indexToStates + 1
        if self.indexUndo == 0: self.redoButton.setEnabled(False)
        self.openState(self.indexToStates)
        self.undoButton.setEnabled(True)        
        
    def selectedToColor(self,id):
        self.saveState() 
        self.selected = self.patternLabel.getSelected()
        for i in range(0,len(self.selected)):
            for x in range(self.selected[i][0],self.selected[i][2]):
                for y in range(self.selected[i][1],self.selected[i][3]):
                    if x >= 0 and x < self.pattern.getWidth() and y >= 0 and y < self.pattern.getHeight():
                         self.pattern.setSquare(x,y,id)    
        if len(self.selected) > 0:
            self.modified = 1    
        self.updatePattern()    
        
    def updatePattern(self):    
        #priprava obrazku na zobrazenie 
        #print("updatujem")
        Ix = self.pattern.getWidth()
        Iy = self.pattern.getHeight() 
        #print(str(Ix)+" "+str(Iy))               
        image = QtGui.QImage(Ix,Iy, QtGui.QImage.Format_ARGB32)
        for y in range(0,Iy):
            for x in range(0,Ix):
                 id = self.pattern.getSquare(x,y)
                 if(id == None): image.setPixel(x,y,QtGui.QColor(255,255,255).rgb()) #??????
                 else: image.setPixel(x,y, self.colorPaletts[id[1]].getColor(id))
        self.patternLabel.setImage(image) #nastavim novy obrazok ktory ma ukazovat         
        #vyrobim nove pouzite farby, ktore dostanu pattern
        self.leftWidget.usedColors = UsedColors(self)
        self.leftWidget.scrollArea.setWidget(self.leftWidget.usedColors)
        self.repaint()          
        
    def cleanSelected(self):
        self.selected = []
         
                 
    @abstractmethod
    def repaint(self): pass
    
    @abstractmethod
    def SaveTo(self,file): pass
    
    @abstractmethod
    def OpenFrom(self,file) : pass
    
    @abstractmethod
    def LabelMoved(self): pass
    

class ImagePatternWidget(AbstractPatternWidget):
    def __init__(self):        
        super(ImagePatternWidget,self).__init__()
      
        self.PathToImage = ""
        self.typ = ""
        
        self.image = QtGui.QImage()
        
        self.settings = ImageSettingsWidget(self)
        self.upLayout.addWidget(self.settings,0,0)
        
        self.leftWidget = ImageLeftWidget(self)
        self.downLayout.addWidget(self.leftWidget,0,0)
        
    def setImage(self,file,type):    
        self.PathToImage = str(file)+"."+str(type) # nastavim cestu k obrazku
        print(self.PathToImage) 
        image = QtGui.QImage(str(self.PathToImage)) # nacitam obrazok
        if image.isNull():
            sprava = "Cannot open "+str(self.PathToImage)
            QtGui.QMessageBox.information(self, "Needle",	sprava)
            return
        self.image = image    
        NImage = image.scaledToHeight(188)
        if NImage.width() > 250: NImage = image.scaledToWidth(250)
        self.settings.setImage(self.image)
        #print(str(NImage.width())+" "+str(NImage.height()))        
        self.leftWidget.miniature.setImage(NImage)
        self.leftWidget.miniature.repaint()    
         
        
    def repaint(self): 
        self.leftWidget.miniature.repaint()       
        self.patternLabel.repaint()       
        
    def SaveTo(self,file):
        print("ukladam")
        #prekopirujem obrazok a symboly 
        pom = self.PathToImage.split(".")
        self.typ = pom[len(pom)-1]
        print(str(file)+"."+str(self.typ))
        if(self.PathToImage != str(file)+"."+str(self.typ)): shutil.copy(self.PathToImage, str(file)+"."+str(self.typ)) #kopirujem iba ak su obrazky rozne  
        
        #vypocitanie cesty
        path = os.path.abspath(__file__) #cesta k prave vykonavanemu programu
        path = path.rstrip("PatternWidget.py") #odstranenie nevhodneho konca
        path = path + "symbols" #prifarenie vhodneho
        print(path)
        
        #prekopirujem znacky
        try:
            print("kopirujem")
            shutil.copytree(path,str(file)+"-symbols") #osetrit co sa ma robit, ked cesta existuje
        except FileExistsError:
            print("najprv mazem")
            shutil.rmtree(str(file)+"-symbols")        #ak existuje, najprv vymazem a potom prekopirujem novu verziu symbolov (mohli sa casom zmenit)
            shutil.copytree(path,str(file)+"-symbols")                      
        
        #ulozenie
        output = open(str(file)+".pt","w")
        output.write(str(NUMBER_OF_IMAGE_PATTERN())+"\n") #to ze ukladam predlohu z obrazka
        output.write(str(self.typ)+"\n") # akeho typu je obrazok
        output.write(str(len(self.pattern.getColors()))+"\n") #dlzka legendy
        # legenda: farba a k nej priradeny znak
        for id in self.pattern.getColors():
            output.write(str(id[0])+" "+str(id[1])+" "+str(self.pattern.getColorSymbol(id))+"\n")
            print(str(id[0])+" "+str(id[1])+" "+str(self.pattern.getColorSymbol(id)))
        
        #cela matica predlohy    
        output.write(str(self.pattern.getWidth())+" "+str(self.pattern.getHeight())+"\n")  # najprv rozmery
        for y in range(0,self.pattern.getHeight()):
            for x in range(0,self.pattern.getWidth()):
                id = self.pattern.getSquare(x,y) 
                if id == None: output.write("N N")
                else: output.write(str(id[0])+" "+str(id[1]))
                if x != self.pattern.getWidth()-1: output.write(" ") 
            output.write("\n")

        self.modified = 0 #ulozili sme...            
    
    def OpenFrom(self,file): 
        self.modified = 0
        input = open(str(file)+".pt")
        line = input.readline()
        line = input.readline()
        self.typ = line.rstrip("\n")
        print(line)
        line = input.readline()
        line = line.rstrip("\n")
        
        colorToSym = {} #načítam legendu
        poc = int(line)
        for i in range(0,poc):
            line = input.readline()
            line = line.rstrip("\n")
            print(line)
            color = line.split(" ")
            colorToSym[(color[0],color[1])] = int(color[2])  
                    
        line = input.readline() # matica predlohy
        line = line.rstrip("\n")
        rozm = line.split(" ")
        width = int(rozm[0])
        height = int(rozm[1])
        self.pattern = Pattern(width,height)
        for y in range(0,height):
            line = input.readline()
            line = line.rstrip("\n")
            a = line.split(" ")
            for x in range(0,width):
                #print(str(x)+" "+str(y)+" "+a[x*2]+" "+a[x*2+1])
                if a[x*2] == "N": pass
                else: self.pattern.setSquare(x,y,(a[x*2],a[x*2+1]))
       
        for id in colorToSym.keys():
            self.pattern.setSymbol(id,colorToSym[id])
                
        self.updatePattern() #zobrazim novy obrazok         
    
    def moved(self):
        self.leftWidget.miniature.repaint()
        
    def do(self,paletteName,squareSize,maxColorCount,choosePalette,resize,method):
        self.saveState()
        print(method) 
        self.modified = 1
        image = self.image
        
        palette = ColorPalette()
        if(choosePalette == "jednoducho"):
            palette = choose_colors_1(image,self.colorPaletts[paletteName],maxColorCount)
        elif(choosePalette == "vylepsene"):
            palette = choose_colors_2(image,self.colorPaletts[paletteName],maxColorCount)
        elif(choosePalette == "rozdelovanie"):    
            palette = choose_colors_3(image,self.colorPaletts[paletteName],maxColorCount)
        
        newImage = QtGui.QImage()
        newImage = prepare_image_edge(image,squareSize,resize)
        #if(resize == "priemer"):
        #    newImage = prepare_image_diam(image,squareSize)
        #elif(resize == "box filter"):
        #    newImage = prepare_image_edge_diam(image,squareSize)    
            
        if(method == "obycajne"):
            #self.pattern = match_colors(image,palette,squareSize,self.pattern.getColorToSym(),paletteName)
            matrix = [[-1]]
            self.pattern = match_colors_dither_matrix(newImage,palette,paletteName,self.pattern.getColorToSym(),matrix,1)
        elif(method == "Floyd-Steinberg dithering"):
            matrix = [[0,-1,7],[3,5,1]]
            self.pattern = match_colors_dither_matrix(newImage,palette,paletteName,self.pattern.getColorToSym(),matrix,16)
        elif(method == "'false' Floyd-Steinberg"):
            matrix = [[-1,3],[3,2]]
            self.pattern = match_colors_dither_matrix(newImage,palette,paletteName,self.pattern.getColorToSym(),matrix,8)
        elif(method == "Jarvis,Judice,Ninke"):
            matrix = [[0,0,-1,7,5],[3,5,7,5,3],[1,3,5,3,1]]
            self.pattern = match_colors_dither_matrix(newImage,palette,paletteName,self.pattern.getColorToSym(),matrix,48)
        elif(method == "Stucki"):
            matrix = [[0,0,-1,8,4],[2,4,8,4,2],[1,2,4,2,1]]
            self.pattern = match_colors_dither_matrix(newImage,palette,paletteName,self.pattern.getColorToSym(),matrix,42)
        elif(method == "Burkes"):
            matrix = [[0,0,-1,8,4],[2,4,8,4,2]]
            self.pattern = match_colors_dither_matrix(newImage,palette,paletteName,self.pattern.getColorToSym(),matrix,32)
        elif(method == "Atkinson"):
            matrix = [[0,-1,1,1],[1,1,1,0],[0,1,0,0]]
            self.pattern = match_colors_dither_matrix(newImage,palette,paletteName,self.pattern.getColorToSym(),matrix,8)
        elif(method == "Sierra3"):
            matrix = [[0,0,-1,5,3],[2,4,5,4,2],[0,2,3,2]]
            self.pattern = match_colors_dither_matrix(newImage,palette,paletteName,self.pattern.getColorToSym(),matrix,32)
        elif(method == "Sierra2"):
            matrix = [[0,0,-1,4,3],[1,2,3,2,1]]
            self.pattern = match_colors_dither_matrix(newImage,palette,paletteName,self.pattern.getColorToSym(),matrix,16)
        elif(method == "Sierra2-4A"):
            matrix = [[0,-1,2],[1,1]]
            self.pattern = match_colors_dither_matrix(newImage,palette,paletteName,self.pattern.getColorToSym(),matrix,4)
        elif(method == "Stevenson-Arce"):
            matrix = [[0,0,0,-1,0,32],[12,0,26,0,30,0,16],[0,12,0,26,0,12],[5,0,12,0,12,0,5]]
            self.pattern = match_colors_dither_matrix(newImage,palette,paletteName,self.pattern.getColorToSym(),matrix,200)            
        elif(method == "Riemersma-Hilbert"):
            rules = ["1d0r0u3","0r1d1l2","3u2l2d1","2l3u3r0"]
            Riem = Riemersma_dithering(newImage,2,palette,rules)
            self.pattern = Riem.getPattern()
        elif(method == "Riemersma-Pean"):
            rules = ["0r2r0d3l1l3d0r2r0","1l3l1u2r0r2u1l3l1","2r0r2u1l3l1u2r0r2","3l1l3d0r2r0d3l1l3"]
            Riem = Riemersma_dithering(newImage,3,palette,rules)
            self.pattern = Riem.getPattern()
        elif(method == "Riemersma2-Hilbert"):
            rules = ["1d0r0u3","0r1d1l2","3u2l2d1","2l3u3r0"]
            Riem = Riemersma_dithering2(newImage,2,palette,rules,16)
            self.pattern = Riem.getPattern()
        elif(method == "Riemersma2-Pean"):
            rules = ["0r2r0d3l1l3d0r2r0","1l3l1u2r0r2u1l3l1","2r0r2u1l3l1u2r0r2","3l1l3d0r2r0d3l1l3"]
            Riem = Riemersma_dithering2(newImage,3,palette,rules,16)
            self.pattern = Riem.getPattern()     
                                            
        self.updatePattern()        
        
           
    
class OwnPatternWidget(AbstractPatternWidget):
    def __init__(self):
        super(OwnPatternWidget,self).__init__()
        self.settings = OwnSettingsWidget(self)
        self.upLayout.addWidget(self.settings,0,0)
        
        self.leftWidget = OwnLeftWidget(self)
        self.downLayout.addWidget(self.leftWidget,0,0)
        
    def repaint(self):       
        self.patternLabel.repaint()       
        
    def SaveTo(self,file):
        print("ukladam")        
        #vypocitanie cesty
        path = os.path.abspath(__file__) #cesta k prave vykonavanemu programu
        path = path.rstrip("PatternWidget.py") #odstranenie nevhodneho konca
        path = path + "symbols" #prifarenie vhodneho
        
        #prekopirujem znacky
        try:
            shutil.copytree(path,str(file)+"-symbols") #osetrit co sa ma robit, ked cesta existuje
        except FileExistsError:
            shutil.rmtree(str(file)+"-symbols")        #ak existuje, najprv vymazem a potom prekopirujem novu verziu symbolov (mohli sa casom zmenit)
            shutil.copytree(path,str(file)+"-symbols")                      
        
        #ulozenie
        output = open(str(file)+".pt","w")
        output.write(str(NUMBER_OF_OWN_PATTERN())+"\n") #to ze ukladam vlastnu predlohu
        output.write(str(len(self.pattern.getColors()))+"\n") #dlzka legendy
        # legenda: farba a k nej priradeny znak
        for id in self.pattern.getColors():
            output.write(str(id[0])+" "+str(id[1])+" "+str(self.pattern.getColorSymbol(id))+"\n")
            print(str(id[0])+" "+str(id[1])+" "+str(self.pattern.getColorSymbol(id)))
        
        #cela matica predlohy    
        output.write(str(self.pattern.getWidth())+" "+str(self.pattern.getHeight())+"\n")  # najprv rozmery
        for y in range(0,self.pattern.getHeight()):
            for x in range(0,self.pattern.getWidth()):
                id = self.pattern.getSquare(x,y)
                if id == None: output.write("N N")
                else: output.write(str(id[0])+" "+str(id[1]))
                if x != self.pattern.getWidth()-1: output.write(" ") 
            output.write("\n")

        self.modified = 0 #ulozili sme...
        
    def OpenFrom(self,file): 
        self.modified = 0
        input = open(str(file)+".pt")
        line = input.readline()
        
        line = input.readline()
        line = line.rstrip("\n")
        
        colorToSym = {} #načítam legendu
        poc = int(line)
        for i in range(0,poc):
            line = input.readline()
            line = line.rstrip("\n")
            print(line)
            color = line.split(" ")
            colorToSym[(color[0],color[1])] = int(color[2])  
                    
        line = input.readline() # matica predlohy
        line = line.rstrip("\n")
        rozm = line.split(" ")
        width = int(rozm[0])
        height = int(rozm[1])
        self.pattern = Pattern(width,height)
        for y in range(0,height):
            line = input.readline()
            line = line.rstrip("\n")
            a = line.split(" ")
            for x in range(0,width):
                #print(str(x)+" "+str(y)+" "+a[x*2]+" "+a[x*2+1])
                if a[x*2] == "N": pass
                else: self.pattern.setSquare(x,y,(a[x*2],a[x*2+1]))
       
        for id in colorToSym.keys():
            self.pattern.setSymbol(id,colorToSym[id])
                
        self.updatePattern() #zobrazim novy obrazok
    
    def Moved(self): pass 
    
    def do(self,width,height):
        self.saveState() 
        self.pattern = Pattern(width,height)
        self.updatePattern()